local mod_name = minetest.get_current_modname()
local S = minetest.get_translator(mod_name)

local anvil_size = {
    type = "fixed",
    fixed = {-4/16, -8/16, -4/16, 4/16, -2/16, 4/16},
}

minetest.register_node('pmb_cooking:anvil', {
    description = pmb_util.desc(S("Iron Anvil"), "grey"),
    _tt_long_desc = S("Can flatten bars into sheets."),
    groups = { item_anvil = 1, solid = 1, oddly_breakable_by_hand = 2, iron = 1, cooker = 1, cooker_anvil = 1, cracky = 1, },
    drawtype = "mesh",
    use_texture_alpha = "opaque",
    paramtype = "light",
    paramtype2 = "facedir",
    tiles = {"pmb_anvil.png"},
    mesh = "pmb_anvil.b3d",
    sounds = (pmb_sounds and pmb_sounds.default_stone()) or {},
    collision_box = anvil_size,
    selection_box = anvil_size,
    _on_node_update = function(pos, cause, user, count)
        local timer = minetest.get_node_timer(pos)
        if not timer:is_started() then
                timer:start(1.0)
        end
    end,
    _on_cook = function(pos, node)
        minetest.sound_play("pmb_anvil_craft", {
            gain = 0.05,
            pos = pos,
            max_hear_distance = 5,
        })
        for x=0, 10 do
            minetest.add_particle({
                pos = vector.offset(pos, 0, 0.1, 0),
                velocity = vector.new((math.random()-0.5)*3, (math.random())*2, (math.random()-0.5)*3),
                expirationtime = math.random() * 1,
                size = math.random() * 4,
                collisiondetection = false,
                vertical = false,
                texture = "pmb_cooking_particle_sparks.png",
                glow = 14,
            })
        end
    end,
    _cook_group = function(pos)
        local below = minetest.get_node(vector.offset(pos, 0, -1, 0))
        if true then return "anvil" end
    end,
    on_timer = pmb_cooking.cooker_node_timer,
    on_construct = pmb_cooking.cooker_on_construct,
    on_punch = function(pos, node, clicker, itemstack, pointed_thing)
        pmb_cooking.cooker_take_item(pos, node, clicker, itemstack, pointed_thing)
    end,
    on_rightclick = function(pos, node, clicker, itemstack, pointed_thing)
        return pmb_cooking.cooker_give_item(pos, node, clicker, itemstack, pointed_thing)
    end,
    after_place_node = function(pos, placer, itemstack, pointed_thing)
        return pmb_util.rotate_and_place_from(pos, placer, itemstack, pointed_thing, {offset = 0, copy_same_node = false})
    end,
    _on_input_item = function(pos, itemstack)
        return pmb_cooking.cooker_offer_item(pos, itemstack)
    end,
})




-- temporary crafting recipe
if true then
    local s = "pmb_items:iron_bar"
    minetest.register_craft({
        output = "pmb_cooking:anvil",
        recipe = {
            {s,  s, s},
            {"", s, ""},
            {s,  s, s},
        },
    })
end


